extern crate image;
extern crate clap;
#[macro_use]
extern crate colour;

mod reskit;
use std::error::Error;
use clap::{App, Arg, SubCommand};
use reskit::soundtrack::dmf::DmfModule;
use reskit::utility;
use reskit::tileset;

fn run() -> Result<(), Box<dyn Error>> {
    let matches = App::new( "reskit" )
        .version( "0.0.2a" )
        .author( "(c) 2021-2023 Ashley N. <ne0ndrag0n@ne0ndrag0n.com>" )
        .about( "Sega Megadrive resource kit and format converter" )
        .subcommand(
            SubCommand::with_name( "tileset" )
                .about( "Generate a Sega Megadrive VDP tileset + palette from a 15-colour image" )
                .arg_from_usage( "-i, --input=<FILE> 'Specify input image'" )
                .arg_from_usage( "-o, --output=<FILE> 'Specify output file'")
                .arg(
                    Arg::with_name( "FORMAT" )
                        .short( "f" )
                        .long( "format" )
                        .help( "Specify output format for tileset (valid options: bin, inc)")
                        .default_value( "bin" )
                        .takes_value( true )
                )
                .arg(
                    Arg::with_name( "TILEORDER" )
                        .short( "to" )
                        .long( "tile-order" )
                        .help( "Specify tile order for tileset (valid options: tile, sprite" )
                        .default_value( "tile" )
                        .takes_value( true )
                )
        )
        .subcommand(
            SubCommand::with_name( "soundtrack" )
                .about( "Generate an on-console soundtrack from a chiptune tracker module" )
                .arg_from_usage( "-i, --input=<FILE> 'Specify input module'" )
                .arg_from_usage( "-o, --output=<FILE> 'Specify output filename (may output multiple files depending on format)'")
                .arg(
                    Arg::with_name( "TRACKER_FORMAT" )
                        .short( "tf" )
                        .long( "tracker-format" )
                        .help( "Specify tracker module format (valid options: dmf (DefleMask Tracker))")
                        .default_value( "dmf" )
                        .takes_value( true )
                )
                .arg(
                    Arg::with_name( "DRIVER_FORMAT" )
                        .short( "df" )
                        .long( "driver-format" )
                        .help( "Specify console sound driver format (valid options: esf (Echo))")
                        .default_value( "esf" )
                        .takes_value( true )
                )
        )
        .get_matches();

    if let Some( matches ) = matches.subcommand_matches( "tileset" ) {
        let input_filename = matches.value_of( "input" ).ok_or( "expected: input filename (-i,--input)" )?;
        let output_filename = matches.value_of( "output" ).ok_or( "expected: output_filename (-o,--output)" )?;

        tileset::generate(
            input_filename,
            output_filename,
            matches.value_of( "FORMAT" ).unwrap(),
            matches.value_of( "TILEORDER" ).unwrap()
        );

        Ok( () )
    } else if let Some( matches ) = matches.subcommand_matches( "soundtrack" ) {
        let input_filename = matches.value_of( "input" ).ok_or( "expected: input filename (-i,--input)" )?;
        let output_filename = matches.value_of( "output" ).ok_or( "expected: output_filename (-o,--output)" )?;

        let result = DmfModule::from_file( &input_filename )?;

        // TODO !!

        Ok( () )
    } else {
        Err( "no tool provided (try --help)" )?
    }
}

fn main() {
    if let Err( error ) = run() {
        utility::print_error( &format!( "{}", error ) );
    }
}